// CRC256   ,  
// Proteus
// icq: 133575351  email:lawnmower-man@mail.ru 

#include<stdio.h>
#include<string.h>

typedef unsigned long dword;
typedef unsigned char byte;

#define BITS 256	//  

// -... (  )
// 82E2443E6320383A20B8A2A0A1EA91A3CCA99A30C5205038349C82AAA3A8FD27
const dword Polynom[BITS/32]={
	0xA3A8FD27,0x349C82AA,0xC5205038,0xCCA99A30,
	0xA1EA91A3,0x20B8A2A0,0x6320383A,0x82E2443E
};


//!     
class CRC_BIG {
	dword crc_table[BITS/32*256];
	dword *GetAt(int index);
public:
	static void Xor(dword data[BITS/32],const dword polynom[BITS/32]);
	static void Shift1(dword data[BITS/32]);
	static void Shift8(dword data[BITS/32]);
public:
	CRC_BIG();
	void Calc(dword res[BITS/32],byte*,int count);
};


CRC_BIG::CRC_BIG()
{
	int n,k; dword b,*c;
	memset(crc_table,0,sizeof(crc_table));
	for(n=0;n<256;n++) {
		c=GetAt(n); c[0]=n;
		for(k=0;k<8;k++) {
			b=c[0]&1;
			Shift1(c);
			if(b) Xor(c,Polynom);
		}
	}
}


dword *CRC_BIG::GetAt(int index)
{
	return crc_table+index*(BITS/32);
}


void CRC_BIG::Xor(dword data[BITS/32],const dword polynom[BITS/32])
{
	int n;
	for(n=0; n<BITS/32; n++) data[n]^=polynom[n];
}

void CRC_BIG::Shift1(dword data[BITS/32])
{
#ifndef _WIN32
	dword b,c=0;
	int n=BITS/32;
	while(n) {
		n--;
		b=data[n]<<31;
		data[n]=(data[n]>>1)|c;
		c=b;
	}
#else
	_asm {
		clc
		mov     ecx,BITS/32
		mov	    esi,[data]
		add		esi,BITS/8
l1:		lea		esi,[esi-4]
		rcr     dword ptr [esi],1
		loop	l1
	}
#endif
}

void CRC_BIG::Shift8(dword data[BITS/32])
{
	byte *b=(byte*)data;
	memcpy(b,b+1,BITS/8-1); b[BITS/8-1]=0;
}


void CRC_BIG::Calc(dword res[BITS/32],byte* data,int count)
{
	byte b;
	memset(res,-1,BITS/8);
	while(count--) {
		b=(byte)res[0];
		Shift8(res);
		Xor(res,GetAt(b^*data++));
	}
}


//    
void _crc_big(dword res[BITS/32],byte *data,int count) 
{
	int n; dword b;
	memset(res,-1,BITS/8);
	while(count--) {
		res[0]^=*data++;
		for(n=0; n<8; n++) {
			b=res[0]&1;
			CRC_BIG::Shift1(res);
			if(b) CRC_BIG::Xor(res,Polynom);
		}
	}
}


void main()
{
	int size;
	dword res1[BITS/32],res2[BITS/32];
	byte data[]="3.1415926535897932384626433832795";
	static const dword test[]={
		0xed8cd8b7,0x759f2278,0x34e1759b,0xd67ba154,
		0x9da12979,0x5cb14f5f,0x679f92ef,0xb6228e3e
	};
	if(sizeof(dword)!=4 || sizeof(byte)!=1) printf("type error\n\a"),getchar();

	size=strlen((char*)data);
	CRC_BIG crc;
	crc.Calc(res1,data,size);
	_crc_big(res2,data,size);

	if(memcmp(res1,test,BITS/8) || memcmp(res2,test,BITS/8)) printf("Internal error\n\a"),getchar();
	printf("node\n");
}